module Util.ViewParts exposing (..)
-- Html
import Html            exposing (Html, Attribute, div, text, node, button, a, h1, h2, label, figure, input, td, th, tr, table)
import Html.Attributes exposing (class, style, href, rel, id, title, value, src)
import Html.Events     exposing (onClick, onInput)
-- Svg
import Svg            exposing (Svg,svg, image,rect,ellipse,circle,polygon,g)
import Svg.Attributes exposing (width,height, viewBox, version, xlinkHref, x, y, cx, cy,rx,ry, r)
import Svg.Events
-- Url
import Url
-- Json
import Json.Encode
-- Own
import TypeHolder exposing (Model,Msg(..))
import Manager.StateManager exposing (..)
import Manager.ModalManager exposing (Modal(..), ModalMsg(..))
import Util.ShapeManager exposing (..)
import Util.Util         exposing (..)

-- Parts of the VIEW
standardViewPart : List (Html Msg)
standardViewPart = [ link [ rel "stylesheet" -- Bluma
                          , href "https://cdn.jsdelivr.net/npm/bulma@0.8.0/css/bulma.min.css"
                          ] []
                   , link [ rel "stylesheet" -- Animationen
                          , href "https://cdnjs.cloudflare.com/ajax/libs/animate.css/3.7.2/animate.min.css"
                          ] []
                   , link [ rel "stylesheet" -- Icon für DropdownMenue
                          , href "https://use.fontawesome.com/releases/v5.8.1/css/all.css"
                          ] []
                   ]
viewHeader_Buttons : State -> Html Msg
viewHeader_Buttons state =
    div []
        [ div [class "section"]
              [ h1 [class "title"] [text "ShapePicture Editor"] ]
        , div [ style "margin-left" "25px"]
              [  button [ onClick (ManageModal (OpenLoadShapePicture "") "")
                       , class "button is-success is-pulled-left is-size-6"
                       , style "margin-bottom" "15px"
                       , style "margin-right" "15px"
                       ] [ text "Ein ShapePicture laden" ]
              , button [ onClick (ManageModal (OpenManagePicture "" "") "")
                       , class "button is-success is-pulled-left is-size-6"
                       , style "margin-bottom" "15px"
                       , style "margin-right" "15px"
                       ] [ text "Bild auswählen" ]
              , button [ onClick (ManageModal OpenShowJson "")
                       , class "button is-success is-pulled-left is-size-6"
                       , style "margin-bottom" "15px"
                       , style "margin-right" "15px"
                       ] [ text "Json Datei anzeigen" ]
              , button [ onClick (ManageModal OpenShowInput "")
                       , class "button is-success is-pulled-left is-size-6"
                       , style "margin-bottom" "15px"
                       , style "margin-right" "15px"
                       ] [ text "Neue Form hinzüfügen" ]
              , button [ onClick (ManageModal OpenShowTable "")
                       , class "button is-success is-pulled-left is-size-6"
                       , style "margin-right" "15px"
                       , style "margin-bottom" "15px"
                       ] [ text "Tabelle mit Formen zeigen" ]
              , if (eq_String_State "DrawShape" state)
                  then let iShape = getInputShapeFromState state
                       in if (eq_String_Shape "Polygon" iShape)
                            then viewDropDownMenue "Polygon wird gezeichnet" True "angle-down" []
                                   [ a [class "dropdown-item"
                                       , onClick (ManageShapes (AddShape Empty) "")
                                       , href ""
                                       ] [text "Speichern"]
                                   , a [class "dropdown-item"
                                       , onClick (ManageModal CloseModal "")
                                       , href ""
                                       ] [text "Abbrechen"]
                                   ]
                            else viewDropDownMenue
                                   (shapeNameStringToGerman
                                     (shapeNameToString
                                       (getInputShapeFromState state))++" zeichnen: abbrechen  ")
                                   False "times" [onClick (ManageModal CloseModal "")] []
                  else viewDropDownMenue "Neue Form zeichnen" True "angle-down" []
                         [ a [class "dropdown-item"
                             , onClick (OpenDrawShape (Circle "" "" ""))
                             , href ""
                             ] [text "Kreis"]
                         , a [class "dropdown-item"
                             , onClick (OpenDrawShape (Rectangle "" "" "" ""))
                             , href ""
                             ] [text "Rechteck"]
                         , a [class "dropdown-item"
                             , onClick (OpenDrawShape (Ellipse "" "" "" ""))
                             , href ""
                             ] [text "Ellipse"]
                         , a [class "dropdown-item"
                             , onClick (OpenDrawShape (Polygon []))
                             , href ""
                             ] [text "Vieleck"]
                         ]
              ]
        ]
viewImage : Model -> Html Msg
viewImage model =
    let picture = maybePictureToPicture model.picture
        picW = String.fromInt picture.w
        picH = String.fromInt picture.h
        loading =  Html.img [ Html.Events.on "load" (decodeImgLoad ImgLoaded)
                            , src (Url.toString picture.url)
                            ] []
    in
    div [ style "margin-left" "25px", style "margin-top" "15px", class "container"]
        [ if picW=="0" then loading else text ""
        , figure [ class "image"]
                 [ svg [ width "100%", viewBox ("0 0 "++picW++" "++picH), version "1.1"]
                       ( List.concat
                           [[image [ width picW, height picH--, Svg.Attributes.preserveAspectRatio "xMinYMin"
                                   , xlinkHref (Url.toString picture.url)
                                   ] []
                            ]
                           , viewSvgShapes model.saved model.svgShapeHover 1
                           , case model.state of
                               Standby         -> []
                               ActiveModal _   -> []
                               DrawShape iShape mPos ->
                                 [ case iShape of
                                     Empty             -> rect [] []
                                     Circle    x y rad ->
                                       if x=="" then circle [cx (String.fromInt mPos.x)
                                                            , cy (String.fromInt mPos.y), r "5"] []
                                                else circle [cx x, cy y, r (abstand x y mPos)] []
                                     Ellipse a b w h ->
                                       if a=="" then ellipse [ cx (String.fromInt mPos.x)
                                                          , cy (String.fromInt mPos.y)
                                                          , rx "10", ry "10"] []
                                                else let coords = manageEllipseCoords
                                                                    {cx= a
                                                                    ,cy= b
                                                                    ,rx= (abstand1D mPos.x (stringToInt a))
                                                                    ,ry= (abstand1D mPos.y (stringToInt b))
                                                                    }
                                                     in  ellipse [ cx coords.cx
                                                                 , cy coords.cy
                                                                 , rx coords.rx
                                                                 , ry coords.ry
                                                                 ] []
                                     Rectangle a b w h ->
                                       if a=="" then rect [ x (String.fromInt mPos.x)
                                                          , y (String.fromInt mPos.y)
                                                          , width "10", height "10"] []
                                                else let coords = manageRectCoords
                                                                    {x= a
                                                                    ,y= b
                                                                    ,w= (abstand1D mPos.x (stringToInt a))
                                                                    ,h= (abstand1D mPos.y (stringToInt b))
                                                                    }
                                                     in  rect [ x      coords.x
                                                              , y      coords.y
                                                              , width  coords.w
                                                              , height coords.h
                                                              ] []
                                     Polygon   pList   ->
                                       case pList of
                                         []    -> circle   [ cx (String.fromInt mPos.x)
                                                           , cy (String.fromInt mPos.y), r "5"
                                                           ] []
                                         p::[] -> Svg.line [ Svg.Attributes.x1 (Tuple.first  p)
                                                           , Svg.Attributes.y1 (Tuple.second p)
                                                           , Svg.Attributes.x2 (String.fromInt mPos.x)
                                                           , Svg.Attributes.y2 (String.fromInt mPos.y)
                                                           , Svg.Attributes.style "stroke:#000;stroke-width:2"
                                                           ] []
                                         p::ps -> polygon [ Svg.Attributes.points
                                                            (shapeCoordsToHtmlCoords
                                                              (Polygon (( String.fromInt mPos.x
                                                                        , String.fromInt mPos.y
                                                                        )::pList
                                                                       )
                                                              )
                                                            )
                                                          ] []
                                 , rect [ x "0", y "0", width picW, height picH
                                        , Svg.Attributes.style
                                            "fill:blue;stroke:pink;stroke-width:5;fill-opacity:0.1;stroke-opacity:0.9"
                                        , onMouseMove UpdateMousePos
                                        , Svg.Events.onClick InputDrawShape
                                        ] []
                                 ]
                           ]
                       )

                 ]
        ]
viewNewShapeInputModal : Shape -> String -> String -> Html Msg
viewNewShapeInputModal iShape ix iy =
    modalCard
      [ modalHeader "Neue Form hinzufügen"
      , modalCardBody
          (List.append
            [ viewDropDownMenue (if (shapeNameToString iShape)==""
                                   then "Form auswählen"
                                   else (shapeNameStringToGerman(shapeNameToString iShape))
                                ) False "angle-down" []
                [ a [ class "dropdown-item"
                    , onClick (ManageModal (Check "Circle") "")
                    , href ""
                    ] [text "Kreis"]
                , a [ class "dropdown-item"
                    , onClick (ManageModal (Check "Rectangle") "")
                    , href ""
                    ] [text "Rechteck"]
                , a [ class "dropdown-item"
                    , onClick (ManageModal (Check "Ellipse") "")
                    , href ""
                    ] [text "Ellipse"]
                , a [ class "dropdown-item"
                    , onClick (ManageModal (Check "Polygon" ) "")
                    , href ""
                    ] [text "Vieleck"]
                ]
            , if (shapeNameToString iShape)==""
                then div [style "margin-top" "120px"] []
                else div [] []
            ]
            (case iShape of
             Empty             -> []
             Circle    a b c   ->
               [ div []
                     [ label [class "label"] [text "Mittelpunkt-X-Position"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "Mittelpunkt-X-Position"
                                        , value a, onInput (ManageModal (Input '1' "")) ] [] ]
                     , label [class "label"] [text "Mittelpunkt-Y-Position"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "Mittelpunkt-Y-Position"
                                        , value b, onInput (ManageModal (Input '2' "")) ] [] ]
                     , label [class "label"] [text "Radius"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "Radius"
                                        , value c, onInput (ManageModal (Input '3' "")) ] [] ]
                     ]
               ]
             Rectangle a b c d ->
               [ div []
                     [ label [class "label"] [text "X-Position"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "X-Position"
                                        , value a, onInput (ManageModal (Input '1' "")) ] [] ]
                     , label [class "label"] [text "Y-Position"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "Y-Position"
                                        , value b, onInput (ManageModal (Input '2' "")) ] [] ]
                     , label [class "label"] [text "Breite"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "Breite"
                                        , value c, onInput (ManageModal (Input '3' "")) ] [] ]
                     , label [class "label"] [text "Höhe"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "Höhe"
                                        , value d, onInput (ManageModal (Input '4' "")) ] [] ]
                     ]
               ]
             Ellipse a b c d ->
               [ div []
                     [ label [class "label"] [text "Mittelpunkt-X-Position"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "Mittelpunkt-X-Position"
                                        , value a, onInput (ManageModal (Input '1' "")) ] [] ]
                     , label [class "label"] [text "Mittelpunkt-Y-Position"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "Mittelpunkt-Y-Position"
                                        , value b, onInput (ManageModal (Input '2' "")) ] [] ]
                     , label [class "label"] [text "X-Radius"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "X-Radius"
                                        , value c, onInput (ManageModal (Input '3' "")) ] [] ]
                     , label [class "label"] [text "Y-Radius"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "Y-Radius"
                                        , value d, onInput (ManageModal (Input '4' "")) ] [] ]
                     ]
               ]
             Polygon   _       ->
               [ div []
                     [ label [class "label"] [text "X-Position"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "X-Position"
                                        , value ix, onInput (ManageModal (Input '1' "")) ] [] ]
                     , label [class "label"] [text "Y-Position"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "Y-Position"
                                        , value iy, onInput (ManageModal (Input '2' "")) ] [] ]
                     , label [class "label"] [text "Eingegebene Vieleck Koordinaten: "]
                     , text (shapeCoordsToString iShape)
                     , div [] [ button [ if isNumeric ix && isNumeric iy
                                           then onClick (ManageModal InputPoint "")
                                           else Html.Attributes.disabled True
                                       , class "button is-success is-size-7"
                                       , style "margin-top" "10px"
                                       ] [ text "Punkt hinzufügen" ]
                              ]
                     ]
               ]
            )
          )
      , modalFooter [ button [ class "button is-success"
                             , if isCorrectFilled iShape
                                 then onClick (ManageShapes (AddShape Empty) "")
                                 else Html.Attributes.disabled True
                             ] [text "Form hinzufügen"]
                    ] 27.0
      ]
viewTableModal : List (Shape, ShapeStyle) -> Html Msg
viewTableModal modelSaved =
    modalCard
      [ modalHeader "Tabelle mit Formen"
      , modalCardBody
          ( if List.length modelSaved == 0
              then [ div [] [text "Es sind keine Formen vorhanden!"] ]
              else [ table [class "table"]
                           ( List.append [ tr []
                                              [ th [] [text "Name/ID"]
                                              , th [] [text "Form"]
                                              , th [] [text "Koordinaten"]
                                              , th [] [text ""]
                                              , th [] [text ""]
                                              ]
                                         ]
                                         ( writeShapeToTable modelSaved 1)
                           )
                   ]
          )
      , modalFooter [] 0
      ]
viewShapeModal : Int -> Int -> ShapeStyle -> Html Msg
viewShapeModal id edit shapeStyle =
    if id <= 0 then div [] [] -- ERROR
               else modalCard [  modalStrongHeader [Html.Events.onDoubleClick (ManageModal (OpenShowShape id 1) "")
                                                   ,Html.Events.onMouseLeave  (ManageModal (OpenShowShape id 0) "")
                                                   ] [
                                                    if edit==1
                                                      then label [] [ input [ class "input"
                                                                            , value shapeStyle.title
                                                                            , onInput (ManageShapes (SaveShapeStyle id shapeStyle "T" ""))
                                                                            ] []
                                                                    ]
                                                      else text (if shapeStyle.title=="default"
                                                                   then ("Form "++String.fromInt id)
                                                                   else shapeStyle.title
                                                                )
                                                   ]
                                , modalCardStrongBody [Html.Events.onDoubleClick (ManageModal (OpenShowShape id 2) "")
                                                      ,Html.Events.onMouseLeave  (ManageModal (OpenShowShape id 0) "")
                                                      ] [
                                                       if edit==2
                                                         then Html.textarea [ class "textarea has-fixed-size"
                                                                            , value shapeStyle.body
                                                                            , onInput (ManageShapes (SaveShapeStyle id shapeStyle "B" ""))
                                                                            ] []
                                                         else if shapeStyle.body=="default"
                                                                then text ("Form "++String.fromInt id++" wurde gefunden!")
                                                                else case Url.fromString shapeStyle.body of
                                                                       Just url -> Html.iframe [ src shapeStyle.body
                                                                                               , style "width"  "100%"
                                                                                               , style "height" "100%"] []
                                                                       Nothing  -> let lines = String.split "\n" shapeStyle.body
                                                                                       writeLines lines2 =
                                                                                         case lines2 of
                                                                                           []   -> []
                                                                                           x::xs-> (div [] [text x, Html.br [] []])::(writeLines xs)
                                                                                   in div [] (writeLines lines)
                                                      ]
                                , modalFooter [ shapeEditButton   id shapeStyle "6"
                                              , shapeDeleteButton id "6"
                                              ] 49.1
                                ]
viewEditShapeModal : Int -> ShapeStyle -> Html Msg
viewEditShapeModal id shapeStyle =
    if id <= 0 then div [] [] -- ERROR
               else modalCard
                      [ modalHeader ("Form "++(if shapeStyle.title=="default"
                                                 then String.fromInt id
                                                 else shapeStyle.title)++" bearbeiten")
                      , modalCardBody [ div [class "control"]
                                            [ table [class "table"]
                                                    [ tr [] [ th [] [text "Optionen"]
                                                            , th [] [text "Werte"]
                                                            ]
                                                    , tr [] [ td [] [ Html.textarea [class "textarea has-fixed-size is-medium"
                                                                                    , Html.Attributes.rows 10
                                                                                    , Html.Attributes.cols 20
                                                                                    , Html.Attributes.readonly True
                                                                                    ] [
                                                                                      text "hoveredFillColor    \n"
                                                                                    , text "hoveredFillOpacity  \n"
                                                                                    , text "hoveredStrokeColor  \n"
                                                                                    , text "hoveredStrokeWidth  \n"
                                                                                    , text "hoveredStrokeOpacity\n"
                                                                                    , text "unHoveredFillColor  \n"
                                                                                    , text "unHoveredFillOpacity\n"
                                                                                    , text "unHoveredStrokeColor\n"
                                                                                    , text "unHoveredStrokeWidth\n"
                                                                                    , text "unHoveredStrokeOpacity"
                                                                                    ]
                                                                    ]
                                                            , td [] [ Html.textarea [ class "textarea has-fixed-size is-medium"
                                                                                    , Html.Attributes.rows 10
                                                                                    , Html.Attributes.cols 10
                                                                                    , onInput (ManageModal (OpenEditShape id shapeStyle "TA" ""))
                                                                                    ] [
                                                                                      text (shapeStyle.hoveredFillColor    ++"\n")
                                                                                    , text (shapeStyle.hoveredFillOpacity  ++"\n")
                                                                                    , text (shapeStyle.hoveredStrokeColor  ++"\n")
                                                                                    , text (shapeStyle.hoveredStrokeWidth  ++"\n")
                                                                                    , text (shapeStyle.hoveredStrokeOpacity++"\n")
                                                                                    , text (shapeStyle.unHoveredFillColor  ++"\n")
                                                                                    , text (shapeStyle.unHoveredFillOpacity++"\n")
                                                                                    , text (shapeStyle.unHoveredStrokeColor++"\n")
                                                                                    , text (shapeStyle.unHoveredStrokeWidth++"\n")
                                                                                    , text  shapeStyle.unHoveredStrokeOpacity
                                                                                    ]
                                                                    ]
                                                            ]
                                                    ]
                                            ]
                                      ]
                      , modalFooter [ button [ class "button is-success"
                                             , onClick (ManageShapes (SaveEditedShape id shapeStyle) "")
                                             ]
                                             [ text "Speichern"]
                                    , shapeDeleteButton id "6"
                                    ] 41.1
                      ]
viewLoadShapePictureModal : String -> Html Msg
viewLoadShapePictureModal urlString =
    modalCard [ modalHeader "Ein ShapePicture laden"
              , modalCardBody [ label [class "label"] [text "Url zum ShapePicture"]
                              , label [] [ input [class "input", Html.Attributes.placeholder "Url zum ShapePicture"
                                                 , value urlString, onInput (ManageModal (OpenLoadShapePicture "")) ] [] ]
                              , text ("Status: "++(isUrlMessage urlString))
                              ]
              , modalFooter [ button [class "button is-success"
                                        , if (isUrl urlString) then onClick (LoadShapePicture urlString)
                                                               else Html.Attributes.disabled True
                                        ] [text "Übernehmen"]
                               ] 22.0
              ]
viewManagePictureModal : String -> Maybe Url.Url -> Maybe Url.Url -> Int -> Int -> Html Msg
viewManagePictureModal s mayUrl currentUrl w h=
    let picUrlState = case mayUrl of
                        Just url -> "Url ist gültig."
                        Nothing  -> "Ungültige Url."
        w1 = String.fromInt w
        h1 = String.fromInt h
    in modalCard [ modalHeader "Bild auswählen"
                 , modalCardBody [ case currentUrl of
                                     Just curUrl -> text ("Aktuelles Bild: "++(Url.toString curUrl))
                                     Nothing     -> text "Bisher wurde kein Bild ausgewählt"
                                 , label [class "label"] [text "Url zum Bild"]
                                 , label [] [ input [class "input", Html.Attributes.placeholder "Url zum Bild"
                                                    , value s, onInput (ManageModal (OpenManagePicture "i1" "")) ] [] ]
                                 , text ("Status: "++picUrlState)
                                 ]
                 , modalFooter [ button [class "button is-success"
                                        , case mayUrl of
                                            Just url -> onClick (SavePicture (Picture url 0 0))
                                            Nothing  -> Html.Attributes.disabled True
                                        ] [text "Übernehmen"]
                               ] 22.0
                 ]
viewJsonModal : Model -> Html Msg
viewJsonModal model =
    let picture = maybePictureToPicture model.picture
    in modalCard [ modalHeader "Json Datei"
                 , modalCardBody [ div [] [ Html.pre [] [Html.text <| Json.Encode.encode 3 <| (writeAlltoJson model.saved (Url.toString picture.url))] ]
                                 , Html.p [] [text <| model.jsonFile.error]
                                 ]
                 , modalFooter [] 0.0
                 ]
-- Helping Methods for the VIEW
viewDropDownMenue : String -> Bool -> String -> List(Attribute Msg) -> List(Svg Msg) -> Html Msg
viewDropDownMenue header isGreen iconType attributes dropDownItems =
  div (List.append
        [ class "dropdown is-hoverable"
        , style "margin-right" "15px"
        , style "margin-bottom" "15px"
        ] attributes
      )
      [ div [ class "dropdown-trigger"]
            [ button [ class ("button "++(if isGreen then "is-success" else "")
                                       ++" aria-haspopup=true aria-controls=dropdown-menu")
                     ]
                     [ Html.span [] [text header]
                     , Html.span [ class "icon is-small"]
                                 [ Html.i [class ("fas fa-"++iconType++" aria-hidden=true")] [] ]
                     ]
            ]
      , if dropDownItems==[]
          then div [] []
          else div [ class "dropdown-menu role=menu"]
                   [ div [ class "dropdown-content"] dropDownItems ]
      ]
viewSvgShapes : List (Shape, ShapeStyle) -> Int -> Int -> List (Svg Msg)
viewSvgShapes shapes activeShape id =
    case shapes of
      []    -> []
      x::xs -> (let f = Tuple.first  x
                    s = Tuple.second x
                in
                  case f of
                    Empty               -> rect [] []
                    Circle    a b c     -> circle ([cx a, cy b, r c                                  ]
                                                   ++svgShapeStyle s activeShape id) []
                    Rectangle a b c d   -> rect   ([Svg.Attributes.x a, y b, width c, height d       ]
                                                   ++svgShapeStyle s activeShape id) []
                    Ellipse   a b c d   -> ellipse([Svg.Attributes.cx a, cy b, rx c, ry d            ]
                                                   ++svgShapeStyle s activeShape id) []
                    Polygon   pointList -> polygon([Svg.Attributes.points (shapeCoordsToHtmlCoords f)]
                                                   ++svgShapeStyle s activeShape id) []
               )::viewSvgShapes xs activeShape (id+1)
svgShapeStyle : ShapeStyle -> Int -> Int -> List(Svg.Attribute Msg)
svgShapeStyle shapeStyle activeShape id =
    List.concat
    [if activeShape==id
       then [ Svg.Attributes.fill          shapeStyle.hoveredFillColor
            , Svg.Attributes.fillOpacity   shapeStyle.hoveredFillOpacity
            , Svg.Attributes.stroke        shapeStyle.hoveredStrokeColor
            , Svg.Attributes.strokeWidth   shapeStyle.hoveredStrokeWidth
            , Svg.Attributes.strokeOpacity shapeStyle.hoveredStrokeOpacity
            ]
       else [ Svg.Attributes.fill          shapeStyle.unHoveredFillColor
            , Svg.Attributes.fillOpacity   shapeStyle.unHoveredFillOpacity
            , Svg.Attributes.stroke        shapeStyle.unHoveredStrokeColor
            , Svg.Attributes.strokeWidth   shapeStyle.unHoveredStrokeWidth
            , Svg.Attributes.strokeOpacity shapeStyle.unHoveredStrokeOpacity
            ]
    ,[ Svg.Events.onMouseOver (SvgShapeHover id)
     , Svg.Events.onMouseOut  (SvgShapeHover 0)
     , Svg.Events.onClick     (ManageModal (OpenShowShape id 0) "")
     ]
    ]
writeShapeToTable : List (Shape, ShapeStyle) -> Int -> List (Html Msg)
writeShapeToTable shapes id =
    case shapes of
      []    -> []
      x::xs -> let f = Tuple.first  x
                   s = Tuple.second x
               in tr [Html.Attributes.id ("Shape"++String.fromInt id)]
                     [ td [] [text (if s.title=="default"then ("Form"++String.fromInt id) else s.title)]
                     , td [] [text (shapeNameStringToGerman(shapeNameToString f))]
                     , td [] [text (shapeCoordsToString     f)]
                     , td [] [shapeDeleteButton id   "7"]
                     , td [] [shapeEditButton   id s "7"]
                     ]
                  :: (writeShapeToTable xs (id+1))
-- Small view objects
shapeDeleteButton : Int -> String -> Html Msg
shapeDeleteButton id size = button [ class ("button is-danger is-size-"++size)
                                   , onClick (ManageShapes (DeleteShape id) "")] [ text "Form löschen" ]
shapeEditButton : Int -> ShapeStyle -> String -> Html Msg
shapeEditButton id shapeStyle size = button [ class ("button is-size-"++size)
                                            , onClick (ManageModal (OpenEditShape id shapeStyle "" "") "")]
                                            [ text "Form bearbeiten"]
-- Modal view-Elements
modalCard : List (Html Msg) -> Html Msg
modalCard modalParts =
    div [class "modal is-active"]
        [ div [class "modal-background"] []
        , div [class "modal-card"] modalParts
        ]
modalHeader : String -> Html Msg
modalHeader title = modalStrongHeader [] [text title]
modalCardBody : List (Html Msg) -> Html Msg
modalCardBody bodyElements = modalCardStrongBody [] bodyElements
modalFooter : List (Html Msg) -> Float -> Html Msg
modalFooter footElements footElementsWidth =
    Html.footer [class "modal-card-foot"]
                (List.append footElements
                            [ button [ class "button"
                                     , style "margin-left" ((String.fromFloat (83.0-footElementsWidth))++"%")
                                     , onClick (ManageModal CloseModal "")
                                     ][text "Schließen"]
                            ]
                )
modalStrongHeader : List (Attribute Msg) -> List (Html Msg) -> Html Msg
modalStrongHeader attributes children =
    Html.header [class "modal-card-head"]
                [ Html.p ([class "modal-card-title"]++attributes) children
                , button [class "delete aria-label=close", onClick (ManageModal CloseModal "")] []
                ]
modalCardStrongBody : List (Attribute Msg) -> List (Html Msg) -> Html Msg
modalCardStrongBody attributes children = Html.section ([class "modal-card-body"]++attributes) children